package com.chopeks

import com.chopeks.extensions.GLExtensionsPreProcessor
import com.chopeks.glife.GLifeEnumPreprocessor
import com.chopeks.transpiler.KotlinToQspTranspiler
import kotlinx.coroutines.*
import mod.ModEnumPreprocessor
import mod.extensions.ModExtensionsPreProcessor
import org.redundent.kotlin.xml.xml
import java.io.File
import java.util.concurrent.TimeUnit

object ProjectGenerator {
  suspend fun generate(dir: File, targetDir: File, copyTo: File) {
    val locations = File(targetDir, "locations").apply { mkdirs() }
    dir.walkTopDown().toList().map {
      GlobalScope.async {
        KotlinToQspTranspiler(
          arrayOf(
            ModExtensionsPreProcessor(), // mod extension can use normal extensions, so it has to be first
            GLExtensionsPreProcessor(),
            GLifeEnumPreprocessor(),
            ModEnumPreprocessor()
          )
        ).apply {
          if (it.extension == "kts") {
            println("processing: ${it.name}")
            try {
              transpile(it, File(locations, "${it.nameWithoutExtension}.qsrc"))
            } catch (e: Throwable) {
              println("bang: ${it.name}")
              throw e
            }
          }
          // copy qsrc just in case
          if (it.extension == "qsrc") {
            File(locations, "${it.nameWithoutExtension}.qsrc").writeText(it.readText())
          }
        }
      }
    }.awaitAll()
    generateProjectFile(dir, targetDir)

    val gameFile = File(targetDir, "$modName.txt")
    projectFileToGameTxt(targetDir, gameFile)
    val qspFile = File(copyTo, "$modName.qsp")
    generateQspFile(gameFile, qspFile)
    gameFile.delete()
  }

  private fun projectFileToGameTxt(src: File, target: File) {
    val process = ProcessBuilder("python2", "src/main/resources/txtmerge.py", File(src, "locations").absolutePath, target.absolutePath, File(src, "$modName.qproj").absolutePath).start()
    process.inputStream.reader(Charsets.UTF_8).use {
      println(it.readText())
    }
    process.waitFor(10, TimeUnit.MINUTES)
  }

  private fun generateQspFile(src: File, target: File) {
    val process = ProcessBuilder("src/main/resources/txt2gam.exe", src.absolutePath, target.absolutePath).start()
    process.inputStream.reader(Charsets.UTF_8).use {
      println(it.readText())
    }
    process.waitFor(10, TimeUnit.MINUTES)
  }


  private fun generateProjectFile(dir: File, targetDir: File): File {
    return File(targetDir, "$modName.qproj").apply {
      val project = xml("QGen-project") {
        attribute("version", "5.5.0")
        "Structure" {
          dir.walkTopDown().forEach {
            if (it.isDirectory) {
              if (it.listFiles()!!.filter { it.isFile }.isNotEmpty()) {
                if (it.absolutePath != dir.absolutePath) {
                  "Folder" {
                    attribute("name", it.absolutePath.split(dir.absolutePath)[1].substring(1).replace("\\", "/"))
                    it.listFiles()!!.forEach {
                      if (it.extension == "kts") {
                        "Location" {
                          attribute("name", it.nameWithoutExtension)
                        }
                      }
                      if (it.extension == "qsrc") {
                        "Location" {
                          attribute("name", it.nameWithoutExtension)
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
      writeText(project.toString(true))
    }
  }
}