package com.chopeks.extensions

import antlr.*
import com.chopeks.glife.extensions.GLExtensions
import com.chopeks.transpiler.ExternalPreProcessor
import org.antlr.v4.runtime.*
import org.antlr.v4.runtime.tree.ParseTreeWalker
import java.io.File

class GLExtensionsPreProcessor : ExternalPreProcessor {
  private fun getFunctionBody(file: File, function: String): String = file.readLines().let { kfile ->
    val functionName = function.split("(")[0]
    val start = kfile.indexOfFirst { "fun ${file.nameWithoutExtension}.$functionName" in it }
    val end = kfile.subList(start, kfile.size).indexOf("}") + start + 1
    val body = kfile.subList(start, end).let { it.subList(1, it.size - 1) }.joinToString("\n")

    val definedParameters = kfile[start].split("(")[1].split(")")[0].split(",").map { it.trim() }
    val providedParameters = function.split("(")[1].split(")")[0].split(",").map { it.trim().trim('"') }

    var newBody = body
    definedParameters.forEachIndexed { index, param ->
      val paramName = param.split(":")[0].trim()
      // TODO lambdas support?
      newBody = if (paramName.startsWith("vararg")) {
        val varargName = paramName.split(" ")[1].trim()
        val varargParams = providedParameters.subList(index, providedParameters.size).joinToString(", ")
        newBody.replace(varargName, varargParams)
      } else {
        newBody.replace(paramName, providedParameters[index])
      }
    }
    newBody.replaceFirst("return", "").trim()
  }

  private fun expandExtension(usage: String): String {
    val function = usage.trim().substring(0..usage.trim().indexOf(")")).trim()
    val callDef = function.trim().split(".", limit = 2)[1]
    val extension = GLExtensions::class.simpleName
    val file = mutableListOf<File>().apply {
      addAll(File("src/main/kotlin/com/chopeks/glife/extensions/").listFiles()!!.toList())
    }.first { it.nameWithoutExtension == extension }
    return getFunctionBody(file, callDef).trim() + " " + usage.split(function)[1].trim()
  }

  override fun process(srcCode: String): String {
    var code = srcCode
    val tokenList = mutableListOf<IntRange>()
    do {
      val lexer = KotlinLexer(CharStreams.fromString(srcCode))
      val tokens = CommonTokenStream(lexer)
      val parser = KotlinParser(tokens)

//      val log = File("GLExtensionsPreProcessor.txt")
//      log.writeText("")
//      for (i in 0 until tokens.numberOfOnChannelTokens) {
//        log.appendText(tokens[i].toString() + '\n')
//      }

      ParseTreeWalker().apply {
        walk(ExtensionCallTracker(tokenList), parser.script());parser.reset()
      }
      tokenList.reversed().forEach {
        val call = code.substring(it)
        val newCall = expandExtension(call)
        code = code.replaceRange(it, newCall)
      }
      tokenList.clear()
    } while (tokenList.isNotEmpty())
    return code
  }

  private class ExtensionCallTracker(val list: MutableList<IntRange>) : KotlinParserBaseListener() {
    private var token: Token? = null
    private var nameToken: Token? = null
    override fun enterImportHeader(ctx: KotlinParser.ImportHeaderContext?) {
      token = null
      nameToken = null
    }

    override fun enterSimpleIdentifier(ctx: KotlinParser.SimpleIdentifierContext) {
      if (token?.text == "ext") {
        nameToken = ctx.start
      }
      if (ctx.text == "ext") {
        token = ctx.start
      }
    }

    override fun enterCallSuffix(ctx: KotlinParser.CallSuffixContext) {
      nameToken?.apply {
        if (ctx.annotatedLambda().isEmpty() && token != null) {
          list.add(token!!.startIndex..ctx.stop.stopIndex)
        }
      }
      token = null
      nameToken = null
    }
  }
}
